;*******************************************************************************
;*
;*      Titel:  LCD-Bibliothek: Unterprogramme zur Ansteuerung eines 2-zeiligen
;*              LC-Displays (16 Zeichen/Zeile, Controller-Typ HD44780) mit 4 Bit
;*              (SR_LCD_2x16_4Bit.asm)
;*
;*      Datum:  03/11/12		Version:        1.1
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*
;*      Prozessor:   ATmega	       Quarzfrequenz: 16MHz
;*      Ausgaenge:   D4-D7 an Bit0-Bit3, RS(Function Select) an Bit 4
;*                   E (Taktleitung) an Bit5, R//W an Masse
;*                   
;*	Informationen zur Funktionsweise:
;*                   
;*      Es muss eine Zeitschleifenbibliothek mit den Unterprogrammen W100us,
;*      W500us, W5ms und W100ms eingebunden werden
;*      
;*      Unterprogramme:
;*      LCDINI:    Initialisiert das LCD-Interface für 4 Bit
;*      LCDSCO:    Sendet einen Befehl zum LCD
;*      LCDOUT:    Sendet ein Zeichen zum LCD
;*      LCDSTF:    Sendet einen Text-String aus dem Flash zum LCD
;*      LCDSTR:    Sendet einen Text-String aus dem SRAM zum LCD
;*      LCDCLR:    Loescht das Display
;*      DEFCHR:    Definiert ein Anwender-Sonderzeichen
;*
;*      Copyright (c) 2010   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*
;*******************************************************************************
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EQU	LCDPORT = PORTB
.EQU	LCDDDR = DDRB
.EQU	LowNib = 1		;LowNib = 1: D4-D7 an Pin0-Pin3
				;LowNib = 0  D4-D7 an Pin4-Pin7
.EQU    RS = 4			;Function Select (0 = Befehl, 1 = Daten)
.EQU    ENA =  5                ;Taktleitung

;-------------------------------------------------------------------------------
;	Initialisierung des HD44780 mit 4-Bit-Schnittstelle
;-------------------------------------------------------------------------------
.IF LowNib == 1
LCDINI: push    r16
	push	r24

        in      r16,LCDDDR      ;6 Ausgaenge initialisieren
        ori     r16,0x3F
        out     LCDDDR,r16
        rcall	W100ms          ;Warte bis Diplaycontroller hochgefahren ist

        ;zuerst in den 8-Bit Modus um nach Reset keinen Bloedsinn anzuzeigen        
        in      r16,LCDPORT     ;Befehl 0x30: Initialisierung 8 Bit (3x) und 5ms
        ori 	r16,0x03	;warten (Nibble verdreht, siehe Anschluss)
        andi    r16,0xC3
        out     LCDPORT,r16     
	rcall	ENACLK          ;Taktimpuls senden        
        rcall   W5ms

	in      r16,LCDPORT	;Befehl wiederholen (bessere Synchronisation)
	ori     r16,0x03        ;warten (Nibble verdreht, siehe Anschluss)
        andi    r16,0xC3
        out     LCDPORT,r16
        rcall   ENACLK          ;Taktimpuls senden

        in      r16,LCDPORT     ;Befehl wiederholen
        ori     r16,0x03        ;warten (Nibble verdreht, siehe Anschluss)
        andi    r16,0xC3
        out     LCDPORT,r16
        rcall   ENACLK          ;Taktimpuls senden

        ;dann in den 4 Bit-Modus
	in      r16,LCDPORT     ;Befehl 0x20: Initialisierung 4 Bit
        ori     r16,0x02        ;warten (Nibble verdreht, siehe Anschluss)
        andi    r16,0xC2
        out     LCDPORT,r16
        rcall   ENACLK          ;Taktimpuls senden

	ldi	r24,0x28	;Befehl 0x28: 4 Bit Daten, 2 Zeilen, Matrix 5x8
	rcall	LCDSCO		;

	ldi	r24,0x0F	;0x0C: Display, Cursor, Blinken eingeschaltet
	rcall	LCDSCO		;

	ldi	r24,0x06	;0x06: Eingabemodus = Adresse inkrem., kein Shift
	rcall	LCDSCO		;Befehl senden

        ldi     r24,0x0C        ;0x0C: Display on, Cursor off, Blinken off
        rcall   LCDSCO

        pop	r24
	pop     r16
	ret
.ELSE
LCDINI: push    r16
	push	r24

        in      r16,LCDDDR      ;6 Ausgaenge initialisieren
        ori     r16,0xFC
        out     LCDDDR,r16
        rcall	W100ms          ;Warte bis Diplaycontroller hochgefahren ist

        ;zuerst in den 8-Bit Modus um nach Reset keinen Bloedsinn anzuzeigen        
        in      r16,LCDPORT     ;Befehl 0x30: Initialisierung 8 Bit (3x) und 5ms
        ori 	r16,0x30	;warten 
        andi    r16,0x33
        out     LCDPORT,r16     
	rcall	ENACLK          ;Taktimpuls senden        
        rcall   W5ms

	in      r16,LCDPORT	;Befehl wiederholen (bessere Synchronisation)
	ori     r16,0x30        ;warten 
        andi    r16,0x33
        out     LCDPORT,r16
        rcall   ENACLK          ;Taktimpuls senden

        in      r16,LCDPORT     ;Befehl wiederholen
        ori     r16,0x30        ;warten 
        andi    r16,0x33
        out     LCDPORT,r16
        rcall   ENACLK          ;Taktimpuls senden

        ;dann in den 4 Bit-Modus
	in      r16,LCDPORT     ;Befehl 0x20: Initialisierung 4 Bit
        ori     r16,0x20        ;warten 
        andi    r16,0x23
        out     LCDPORT,r16
        rcall   ENACLK          ;Taktimpuls senden

	ldi	r24,0x28	;Befehl 0x28: 4 Bit Daten, 2 Zeilen, Matrix 5x8
	rcall	LCDSCO		;

	ldi	r24,0x0F	;0x0C: Display, Cursor, Blinken eingeschaltet
	rcall	LCDSCO		;

	ldi	r24,0x06	;0x06: Eingabemodus = Adresse inkrem., kein Shift
	rcall	LCDSCO		;Befehl senden

        ldi     r24,0x0C        ;0x0C: Display on, Cursor off, Blinken off
        rcall   LCDSCO

        pop	r24
        pop     r16
	ret
.ENDIF

;-------------------------------------------------------------------------------
;       Taktimpuls senden
;-------------------------------------------------------------------------------
ENACLK: sbi     LCDPORT,ENA       ;Taktsignal (High Impuls senden)
        rcall   W100us
        cbi     LCDPORT,ENA
        rcall   W500us
        ret

;-------------------------------------------------------------------------------
;	Befehl zum LCD senden (RS = 0, Befehl in r24)
;-------------------------------------------------------------------------------
.IF LowNib == 1
LCDSCO: push    r16
        push    r17

	mov     r16,r24         ;HighNibble ausmaskieren und ausgeben ohne
        swap    r16             ;andere Bits zu beeinflussen
        andi    r16,0x0F
        in      r17,LCDPORT
        or      r17,r16
        ori     r16,0xF0
        and     r17,r16
        cbr     r17,1<<RS       ;Function Select = Befehl
        out     LCDPORT,r17
        rcall   ENACLK          ;Taktimpuls senden        
        andi    r24,0x0F        ;LowNibble ausmaskieren und ausgeben ohne
        in      r17,LCDPORT     ;andere Bits zu beeinflussen
        or      r17,r24
        ori     r24,0xF0
        and     r17,r24
        cbr     r17,1<<RS       ;Function Select = Befehl
        out     LCDPORT,r17
        rcall   ENACLK          ;Taktimpuls senden

	pop     r17
        pop     r16
        ret
.ELSE
LCDSCO: push    r16
        push    r17

	mov     r16,r24         ;HighNibble ausmaskieren und ausgeben ohne        
        andi    r16,0xF0        ;andere Bits zu beeinflussen
        in      r17,LCDPORT
        or      r17,r16
        ori     r16,0x0F
        and     r17,r16
        cbr     r17,1<<RS       ;Function Select = Befehl
        out     LCDPORT,r17
        rcall   ENACLK          ;Taktimpuls senden        
        andi    r24,0x0F        ;LowNibble ausmaskieren und ausgeben ohne
        swap    r24             ;andere Bits zu beeinflussen
        in      r17,LCDPORT 
        or      r17,r24
        ori     r24,0x0F
        and     r17,r24
        cbr     r17,1<<RS       ;Function Select = Befehl
        out     LCDPORT,r17
        rcall   ENACLK          ;Taktimpuls senden

	pop     r17
        pop     r16
        ret
.ENDIF

;-------------------------------------------------------------------------------
;	Datenbyte zum LCD senden (RS = 1, Daten in r24)
;-------------------------------------------------------------------------------
.IF LowNib == 1
LCDOUT: push    r16
        push    r17

        mov     r16,r24         ;HighNibble ausmaskieren und ausgeben ohne
        swap    r16             ;andere Bits zu beeinflussen
        andi    r16,0x0F
        in      r17,LCDPORT
        or      r17,r16
        ori     r16,0xF0
        and     r17,r16
        sbr     r17,1<<RS       ;Function Select = Daten
        out     LCDPORT,r17
        rcall   ENACLK          ;Taktimpuls senden
        andi    r24,0x0F        ;LowNibble ausmaskieren und ausgeben ohne
        in      r17,LCDPORT     ;andere Bits zu beeinflussen
        or      r17,r24
        ori     r24,0xF0
        and     r17,r24
        sbr     r17,1<<RS       ;Function Select = Daten
        out     LCDPORT,r17
        rcall   ENACLK          ;Taktimpuls senden

        pop     r17
        pop     r16
        ret
.ELSE
LCDOUT: push    r16
        push    r17

        mov     r16,r24         ;HighNibble ausmaskieren und ausgeben ohne
        andi    r16,0xF0        ;andere Bits zu beeinflussen
        in      r17,LCDPORT
        or      r17,r16
        ori     r16,0x0F
        and     r17,r16
        sbr     r17,1<<RS       ;Function Select = Daten
        out     LCDPORT,r17
        rcall   ENACLK          ;Taktimpuls senden
        andi    r24,0x0F        ;LowNibble ausmaskieren und ausgeben ohne
        swap	r24             ;andere Bits zu beeinflussen
        in      r17,LCDPORT     
        or      r17,r24
        ori     r24,0x0F
        and     r17,r24
        sbr     r17,1<<RS       ;Function Select = Daten
        out     LCDPORT,r17
        rcall   ENACLK          ;Taktimpuls senden

        pop     r17
        pop     r16
        ret
.ENDIF        

;-------------------------------------------------------------------------------
;	TextAusgabe FLASH: Gibt Zeichen an das Display bis Nullbyte
;	Adresse des ersten Zeichens wird mit Z-Pointer übergeben.
;-------------------------------------------------------------------------------
LCDSTF: push	r24
LCDSFL:	lpm     r24,Z+
        cpi     r24,0
        breq    LCDSFR
	rcall	LCDOUT		;Zeichen senden, Autoinkrement der Adresse
        rjmp	LCDSFL		;alle Zeichen an das LCD ausgeben
LCDSFR:	pop	r24
	ret

;-------------------------------------------------------------------------------
;       TextAusgabe RAM: Gibt Zeichen an das Display bis Nullbyte
;       Adresse des ersten Zeichens wird mit Z-Pointer übergeben.
;-------------------------------------------------------------------------------
LCDSTR: push	r24
LCDSRL:	ld      r24,Z+
        cpi     r24,0
        breq    LCDSRR
        rcall   LCDOUT          ;Zeichen senden, Autoinkrement der Adresse
        rjmp    LCDSRL          ;alle Zeichen an das LCD ausgeben
LCDSRR: pop	r24
	ret


;-------------------------------------------------------------------------------
;       Display loeschen
;-------------------------------------------------------------------------------
LCDCLR: push	r24
	ldi     r24,0x01        ;0x01: Display Clear
        rcall   LCDSCO          ;
        rcall   W5ms
        pop	r24
        ret

;-------------------------------------------------------------------------------
;       Definition eigener Sonderzeichen (Zeichencode in r24)
;       
;       ueberarbeiten!
;-------------------------------------------------------------------------------
DEFCHR: push    r16
        lsl	r24		;r24 mit 2 multipliziert
	lsl	r24		;r24 mit 4 multipliziert
	lsl	r24		;r24 mit 8 multipl.
 			;(8 Zeilen pro Zeichen CG-RAM)
	mov	ZL,r24		;Z-Ptr zeigt auf Anf.-Adr. des Zeichens in der
				;Char.-Tab 
	ldi	ZH,high(CHRTAB);High-Byte der Punktemuster-Tabellenadr.
	rol	ZH		;Z-Ptr um 1 Pos. nach links schieben
	sbr	r24,1<<6	;Bit6 setzen -> Zieladr. ist im CG-RAM
	
	rcall	LCDSCO		;Adressenzähler im CG-RAM setzen
	ldi	r16,8
DC1:	lpm			;Punktemuster nach R0 laden
	mov	r24,R0		;Punktemuster nach r24 kopieren
	
	rcall	LCDOUT		;Zeichenmuster senden, Autoinkrement der Adresse
	adiw	ZL,1		;Z-Ptr inkrementieren
	dec	r16		;Zähler - 1
	brne	DC1		;alle 8 Zeilen ins CG-RAM kopieren
        pop     r16
	ret


CHRTAB:
.DB   $04,$04,$1F,$04,$04,$00,$1F,$00   ; Ch.-Code $00: Zeichen "+/-"
.DB   $0C,$02,$04,$08,$0E,$00,$00,$00   ; Ch.-Code $01: Zeichen "^2"
.DB   $0C,$02,$0C,$02,$0C,$00,$00,$00   ; Ch.-Code $02: Zeichen "^3"
.DB   $18,$06,$01,$06,$18,$00,$1F,$00   ; Ch.-Code $03: Zeichen ">="
.DB   $03,$0C,$10,$0C,$03,$00,$1F,$00   ; Ch.-Code $04: Zeichen "<="
.DB   $04,$04,$1F,$04,$1F,$04,$04,$00   ; Ch.-Code $05: Zeichen "ungleich"
.DB   $00,$1F,$00,$1F,$00,$1F,$00,$00   ; Ch.-Code $06: Zeichen "identisch"
.DB   $15,$15,$15,$15,$15,$15,$15,$00   ; Ch.-Code $06: Zeichen "3 Balken"
TEXT1:
.DB   "   :      /  /  ",0,0
TEXT2:
.DB   "ppm ",0,0
TEXT3:
.DB   0,' ',1,' ',2,' ',3,' ',4,' ',5,' ',6,' ',7,' '
TEXT4:
.DB   "  CO22 mobile   ",0,0
TEXT5:
.DB   "  www.weigu.lu  ",0,0
TEXT6:
.DB   "Cursor-Pos. = 65",0,0
TEXT7:
.DB   "Grad C",0,0
