;*******************************************************************************
;*
;*      Titel:	USB-Bibliothek: Unterprogramme und Interruptroutinen zur Nutzung
;*              des Teensy2 (SR_USB_32u4_CO22.asm)
;*	Datum:	23/01/10	Version:	2.0
;*      Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*
;*	Prozessor:   ATmega32u4	      Quarzfrequenz: 16MHz
;*	Eingaenge:  
;*	Ausgaenge:  
;*
;*	Informationen zur Funktionsweise:
;*      Dokumentation auf www.weigu.lu/b/co22 und www.weigu.lu/b/usb/teensy2
;*
;*      Copyright (c) 2010   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*
;*******************************************************************************

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Variablen in SRAM
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.DSEG                           ;was ab hier folgt kommt in den SRAM-Speicher
;UP INIEP  Parameter
EP_NUM: .BYTE 1                 ;EP NUM: 0-6
EP_TYP: .BYTE 1                 ;EP TYPE: 0=Contr. 1= Iso. 2= Bulk 3 = Inter.
EP_DIR: .BYTE 1                 ;EP DIR: 0=OUT 1=IN
EP_SIZ: .BYTE 1                 ;EP SIZE: 0=8 Byte 1=16 2=32 3=64 (4=128 5=256)
EP_BK:  .BYTE 1                 ;EP BANK: 0=eine Bank 1=doppelte Bank

;Variablen fuer SETUP Paket
SU_R_T:	.BYTE 1 		;bmRequestType
SU_REQ: .BYTE 1 		;bRequest
SU_VAL: .BYTE 2 		;wValue
SU_IND: .BYTE 2 		;wIndex
SU_LEN: .BYTE 2 		;wLength

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Konstanten im Flash
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.CSEG                           ;was ab hier folgt kommt in den FLASH-Speicher
        ;Konstanten fuer Endpoints
.EQU    EP0_T = 0               ;EP0 TYPE: 0=Control
.EQU    EP0_D = 0               ;EP0 DIR: 0=OUT
.EQU    EP0_S = 0               ;EP0 SIZE: 0=8 Byte
.EQU    EP0_B = 0               ;EP0 BANK: 0=1 Bank
.EQU    EP0_FS = 8              ;FIFO Size 8 Byte

.EQU    EP1_T = 2               ;EP1 TYPE: 2= Bulk
.EQU    EP1_D = 0               ;EP1 DIR: 0=OUT
.EQU    EP1_S = 3               ;EP1 SIZE: 3=64 Byte
.EQU    EP1_B = 0               ;EP1 BANK: 0=1 Bank
.EQU    EP1_FS = 64             ;FIFO Size 64 Byte

.EQU    EP2_T = 2               ;EP2 TYPE: 2= Bulk
.EQU    EP2_D = 1               ;EP2 DIR: 1=IN
.EQU    EP2_S = 3               ;EP2 SIZE: 3=64 Byte
.EQU    EP2_B = 1               ;EP2 BANK: 1=2 Baenke
.EQU    EP2_FS = 64             ;FIFO Size 64 Byte

;;; hier erweitern falls weitere EPs benoetigt werden

;Globale Konstanten
.EQU	LANG_I = 0		;Language Descriptor Index
.EQU	MANU_I = 1		;Manufacturer String Index
.EQU	PROD_I = 2		;Product String Index
.EQU	SERI_I = 3		;Serial Number String Index
.EQU	NR_EPS = 2		;Benutzte Endpoints (ohne EP0)

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Interrupts
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;-------------------------------------------------------------------------------
;       USB GENeral Interrupt Service Routine (reagiert nur auf End Of ReSeT)
;-------------------------------------------------------------------------------
ISRGEN: push    r16             ;rette Register (lokale Variablen und SREG)
        in      r16,SREG
        push    r16

        lds     r16,UDINT       ;falls kein End Of ReSeT Interrupt, dann raus
        sbrs    r16,EORSTI
        rjmp    ISRG1
        lds	r16,UDINT	;cbi UDINT,EORSTI (Bit3)
	andi	r16,0xF7        ;Loesche End Of ReSeT Interrupt Flag
	sts	UDINT,r16

        ;Initialisiere EP0
	ldi     r16,0           ;Parameter fuer UP INITEP
        sts     EP_NUM,r16      ;EP_NUM = 0
        ldi     r16,EP0_T
        sts     EP_TYP,r16
        ldi     r16,EP0_D
        sts     EP_DIR,r16
        ldi     r16,EP0_S
        sts     EP_SIZ,r16
        ldi     r16,EP0_B
        sts     EP_BK,r16
        rcall   INIEP

	lds	r16,UEIENX	;sbi UEIENX,RXSTPE (Bit3)
	ori	r16,0x08        ;Erlaube EP COM Interrupts wenn ein neues
	sts	UEIENX,r16	;gueltiges SETUP Packet vorliegt (RXSTPI=1)

ISRG1:  pop     r16             ;Register wiederherstellen
        out     SREG, r16
        pop     r16
        reti

;-------------------------------------------------------------------------------
;       USB EndPoint COMmunication Interrupt Service Routine
;-------------------------------------------------------------------------------
ISRCOM: push    r16             ;rette Register (lokale Variablen und SREG)
        in      r16,SREG
        push    r16

        lds     r16,UEINT       ;EP Nummer durch Bitposition in UEINT
        cpi     r16,0b00000001	;EP0?
        breq    ISREP0
	cpi     r16,0b00000010	;EP1?
        breq    ISREP1
	cpi     r16,0b00000100	;EP2?
        breq    ISREP2

	;;; hier erweitern falls weitere EPs benoetigt werden

	rjmp    ISRCR           ;Raus

ISREP0:	ldi	r16,0		;Waehle EP0
	sts     UENUM,r16
        lds     r16,UEINTX      ;falls kein gueltiges SETUP Paket, dann raus
        sbrs    r16,RXSTPI
        rjmp    ISRCR           ;Raus
        rcall	SETUP		;Unterprogramm SETUP (Enumeration)
        rjmp    ISRCR           ;Raus

ISREP1:	ldi	r16,1		;Waehle EP1
	sts     UENUM,r16
        rcall	EP1_SP		;Unterprogramm EP1_SP (PD4-PD7 (LEDs) schalten)
        rjmp    ISRCR           ;Raus

ISREP2:	ldi	r16,2		;Waehle EP2
	sts     UENUM,r16
        rcall   EP2_AD          ;Unterprogramm EP2_AD (ADC 2 Byte einlesen)
        rjmp    ISRCR           ;Raus

        ;;; hier erweitern falls weitere EPs benoetigt werden

ISRCR:  pop     r16             ;Register wiederherstellen
        out     SREG, r16
        pop     r16
        reti

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Unterprogramme
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;-------------------------------------------------------------------------------
;       UP INIDEV: USB-Aktivierung (Full-Speed 12Mbit/s) und Interrupts erlauben
;-------------------------------------------------------------------------------
INIDEV: push    r16             ;rette Register (lokale Variablen)

        ldi     r16,0x01        ;UHWCON = 0x01, Device Modus und Regelkreis zur
        sts     UHWCON,r16      ;Versorgung der D+ und D- Leitung einschalten
        ldi     r16,0xB0        ;USBCON = 0xB0, USB Sender/Empf. und Takt ein-
        sts     USBCON,r16      ;schalten und VBUS-Leitung einschalten
 	ldi     r16,0x90        ;USBCON = 0x90, FRZCLK=0, noetig, da sonst kein
        sts     USBCON,r16      ;Transfer und somit EORSTI nicht ausloesen kann
	ldi     r16,0xB0        ;USBCON = 0xB0, FRZCLK=1, Strom sparen
        sts     USBCON,r16


        ;ldi     r16,0x6A
        ;out     PLLFRQ,r16


	;Starte PLL
        ldi     r16,0x10        ;PLLCSR = 0x10 Vorteiler fuer 16MHz Quarz
        out     PLLCSR,r16
        ldi     r16,0x12        ;PLLCSR = 0x12, PLL Enable Bit = 1, starte PLL
        out     PLLCSR,r16
INIDE1: in      r16,PLLCSR      ;Warte bis PLOCK = 1
        sbrs    r16,PLOCK
        rjmp    INIDE1
        ldi     r16,0x90        ;USBCON = 0x90, FRZCLK=0, aktiviere den Takt
        sts     USBCON,r16

        clr     r16             ;UDCON = 0 Attach: Verbinde das Device
        sts     UDCON,r16
	ldi     r16,0x08        ;UDIEN = 0x08 (EORSTE = 1)
        sts     UDIEN,r16       ;erlaube End Of ReSeT Interrupt
        sei                     ;Interrupts global erlauben

        pop     r16             ;Register wiederherstellen
        ret

;-------------------------------------------------------------------------------
;       UP INIEP: Aktiviert EP
;       Parameter (jeweils 1 Byte) zur Uebergabe im SRAM (siehe Hauptpr.)
;       EP_NUM 0-6
;       EP_TYP 0=Contr. 1= Iso. 2= Bulk 3 = Inter.
;       EP_DIR 0=OUT 1=IN
;       EP_SIZ 0=8 Byte 1=16 2=32 3=64 (4=128 5=256)
;       EP_BK  0=eine Bank 1=doppelte Bank
;-------------------------------------------------------------------------------
INIEP:  push    r16             ;rette Register (lokale Variablen)
        push    r17

	lds     r16,EP_NUM      ;UENUM = EP_NUM waehle Endpoint
        sts     UENUM,r16
	lds	r16,UECONX	;sbi UECONX,EPEN (Bit 0)
	ori	r16,0x01	;aktiviere EP
	sts	UECONX,r16
        lds     r16,EP_TYP      ;TYPE + DIR festlegen (SF-Reg. UECFG0X)
        ror     r16             ;TYPE in Bit 6 und 7! rotiere 3*rechts durch
        ror     r16             ;carry (kuerzer als 6*links)
        ror     r16
        lds     r17,EP_DIR      ;DIR in Bit 0, andere Bits = 0
        or      r16,r17
        sts     UECFG0X,r16
        lds     r16,EP_SIZ      ;SIZE + BANK festlegen (SF-Reg. UECFG1X)
        lsl     r16             ;SIZ in Bit 4-6 rotiere 4*links
        lsl     r16
        lsl     r16
        lsl     r16
        lds     r17,EP_BK       ;BK in Bit 2-3 rotiere 2*links
        lsl     r17             ;Bit 1 = ALLOC = 0 andere Bits = 0 (reserviert)
        lsl     r17
        or      r16,r17
        sts     UECFG1X,r16
	lds	r16,UECFG1X	;sbi UECFG1X,ALLOC (Bit 1)
	ori	r16,0x02	;Speicher zuweisen
	sts	UECFG1X,r16

	pop     r17             ;Register wiederherstellen
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       UP SETUP: Behandle eine SETUP Anfrage (Paket) vom Host (EP0)
;-------------------------------------------------------------------------------
SETUP:  push    r16             ;rette Register (lokale Variablen)
	push    r17

        ;Lese 8 Byte des SETUP Pakets ins SRAM
        lds	r16,UEDATX      ;8 Setup Byte im SRAM ablegen
	sts	SU_R_T,r16	;bmRequestType
	lds	r16,UEDATX
	sts	SU_REQ,r16	;bRequest
	lds	r16,UEDATX
	sts	SU_VAL,r16	;wValue
	lds	r16,UEDATX
	sts	SU_VAL+1,r16
	lds	r16,UEDATX
	sts	SU_IND,r16	;wIndex
	lds	r16,UEDATX
	sts	SU_IND+1,r16
	lds	r16,UEDATX
	sts	SU_LEN,r16	;wLength
	lds	r16,UEDATX
	sts	SU_LEN+1,r16

	lds	r16,UEINTX	;cbi UEINTX,RXSTPI (Bit 3)
	andi	r16,0xF7	;ACK SU Paket (ISR abschl. und FIFO loeschen)
	sts	UEINTX,r16

	lds	r16,SU_R_T	;Kontrolliere ob ein Standard Device Request
	andi	r16,0x60	;vorliegt, sonst STALL (und raus).
	breq	SETUPT
	rjmp	SETUPS		;nicht unterstuetzter Req. -> STALL (und raus)

SETUPT:	lds	r16,SU_REQ	;Teste Art des Requests
	cpi	r16,0x00
	breq	SETUP0		;Get_Status
	cpi	r16,0x05
	breq	SETUP5		;Set_Adress
	cpi	r16,0x06
	breq	SETUP6		;Get_Descriptor
	cpi	r16,0x09
	brne	SETUTS		;falls nicht, dann raus (zu weit fuer branch)
	rjmp	SETUP9		;Set_Configuration
SETUTS:	rjmp	SETUPS		;raus

SETUP0: 			;Get_Status
	clr r16
	sts	UEDATX,r16
	sts	UEDATX,r16
	lds	r16,UEINTX	;cbi UEINTX,TXINI (Bit 0)
	andi	r16,0xFE	;Paket abschicken
	sts	UEINTX,r16
SETU0L:	lds	r16,UEINTX	;Warte bis RXOUTI gesetzt, ZLP (ACK) vom PC
	andi	r16,0x04	;
	breq	SETU0L		;
	rjmp	SETUPR		;raus

SETUP5: 			;Set_Adress
	lds	r16,SU_VAL	;LSB(wValue) enthaelt bei Set_Adress Geraeteadr.
	andi	r16,0x7F	;0-127
	sts	UDADDR,r16	;Speichere Adresse
	lds	r16,UEINTX	;cbi UEINTX,TXINI (Bit 0)
	andi	r16,0xFE	;sende ZLP Pak. (meldet Erfolg, loescht EP Bank)
	sts	UEINTX,r16
SETU51:	lds	r16,UEINTX	;Warte bis TXINI wieder gesetzt (fertig,
	andi	r16,0x01	;ACK vonm PC, EP Bank wieder frei)
	breq	SETU51
	lds	r16,UDADDR	;sbi UDADDR,ADDEN (Bit 7)
	ori	r16,0x80	;Adresse aktivieren
	sts	UDADDR,r16
	rjmp	SETUPR		;raus
SETUP6:				;Get_Descriptor
	lds	r16,SU_VAL+1	;HIGH(Su_Val) enthaelt Deskriptor Nummer
	cpi	r16,1		;Device Deskriptor
	breq	SETU6D
        cpi	r16,2		;Configuration Deskriptor
	breq	SETU6C
	cpi	r16,3		;String Deskriptor
	breq	SETU6S
	rjmp	SETUPR		;raus

SETU6D: ldi     ZL,LOW(DEVDES*2)
	ldi	ZH,HIGH(DEVDES*2)
	rjmp 	SETU64

SETU6C: lds	r18,SU_LEN	;Wieviele Bytes angefragt?
	ldi	ZL,LOW(CONDES*2)
	ldi	ZH,HIGH(CONDES*2)
	rjmp 	SETU65

SETU6S: lds	r16,SU_VAL	;Enthaelt Sring Index Nummer
	cpi	r16,LANG_I
        breq	SETU60
	cpi	r16,MANU_I	;Configuration Deskriptor
	breq	SETU61
	cpi	r16,PROD_I	;Configuration Deskriptor
	breq	SETU62
	cpi	r16,SERI_I	;Configuration Deskriptor
	breq	SETU63
	rjmp	SETU65
SETU60:	ldi	ZL,LOW(ST0DES*2)
	ldi	ZH,HIGH(ST0DES*2)
	rjmp	SETU64
SETU61:	ldi	ZL,LOW(ST1DES*2)
	ldi	ZH,HIGH(ST1DES*2)
	rjmp	SETU64
SETU62:	ldi	ZL,LOW(ST2DES*2)
	ldi	ZH,HIGH(ST2DES*2)
	rjmp	SETU64
SETU63:	ldi	ZL,LOW(ST3DES*2)
	ldi	ZH,HIGH(ST3DES*2)
SETU64:	lpm	r18,Z
SETU65: rcall	SNDDES
	rjmp	SETUPR		;raus

SETUP9: 			;Set_Configuration
	;Benutzte EPs müssen abgeschaltet werden!
	ldi	r17,NR_EPS	;Zaehler init. um benutzte EPs abzuschalten
SETU9L:	sts	UENUM,r17	;EP auswaehlen
	lds	r16,UECONX	;cbi UECONX,EPEN (Bit 0)
	andi	r16,0xFE	;schalte den EP ab
	sts	UECONX,r16
	lds	r16,UECFG1X	;cbi UECFG1X,ALLOC (Bit 1)
	andi	r16,0xFD	;Speicherbank freigeben
	sts	UECFG1X,r16
	dec	r17
	brne	SETU9L

	;Initialisieren EP1 (OUT EP: Parameter und RXOUT Interrupt erlauben)
	ldi	r16,1           ;Parameter fuer UP INITEP
        sts     EP_NUM,r16      ;EP_NUM = 1
        ldi     r16,EP1_T
        sts     EP_TYP,r16
        ldi     r16,EP1_D
        sts     EP_DIR,r16
        ldi     r16,EP1_S
        sts     EP_SIZ,r16
        ldi     r16,EP1_B
        sts     EP_BK,r16
        rcall   INIEP
	lds	r16,UEIENX	;sbi UEIENX,RXOUTE (Bit 2)
	ori	r16,0x04	;Erlaube RX OUT (EP3) Interrupt (INT wird von HW
	sts	UEIENX,r16	;gesetzt wenn OUT Daten vorhanden sind).
				;Es wird dann ein neuer COM INT ausgeloest.

	;Initialisieren EP2 (IN EP: Parameter und NAKIN Interrupt erlauben)
	ldi	r16,2           ;Parameter fuer UP INITEP
        sts     EP_NUM,r16      ;EP_NUM = 2
        ldi     r16,EP2_T
        sts     EP_TYP,r16
        ldi     r16,EP2_D
        sts     EP_DIR,r16
        ldi     r16,EP2_S
        sts     EP_SIZ,r16
        ldi     r16,EP2_B
        sts     EP_BK,r16
        rcall   INIEP
	lds	r16,UEIENX	;sbi UEIENX,NAKINE (Bit 6)
	ori	r16,0x40	;Erlaube NAK IN (EP1) Interrupt (INT wird von HW
	sts	UEIENX,r16	;gesetzt wenn IN Anfrage von PC mit NAK beantw.
				;wurde). Es wird ein neuer COM INT ausgeloest.

	;;;Initialisieren EP3
        ;;; hier erweitern falls weitere EPs benoetigt werden

	clr	r16		;EP0 auswaehlen
	sts	UENUM,r16
	lds	r16,UEINTX	;cbi UEINTX,TXINI (Bit 0)
	andi	r16,0xFE	;sende ZLP Pak. (meldet Erfolg, loescht EP Bank)
	sts	UEINTX,r16
SETU9M:	lds	r16,UEINTX	;Warte bis TXINI wieder gesetzt (fertig,
	andi	r16,0x01	;ACK vonm PC, EP Bank wieder frei)
	breq	SETU9M
	rjmp	SETUPR

SETUPS: lds	r16,UECONX	;sbi UECONX,STALLRQ	(Bit 5)
	ori	r16,0x20	;kein Standard Request! erfrage STALL Antwort
	sts	UECONX,r16

SETUPR: pop	r17
	pop     r16             ;Register wiederherstellen
        ret

;-------------------------------------------------------------------------------
;       UP WR_DES: Schreibe Deskriptor
;-------------------------------------------------------------------------------
SNDDES:	push	r16		;rette Register (lokale Variablen)
	push	r17

	clr 	r17		;Zaehler1 (geschriebene Bytes) auf 0
SNDDE1: lds	r16,UEINTX	;Unterbrechung vom PC (Host) dann raus
	andi	r16,0x04	;RXOUTI
	brne	SNDDER
	;jeweils nach 8 Bytes das Paket absenden und Speicherbank loeschen
	lpm	r16,Z+		;Byte ins FIFO
	sts	UEDATX,r16
	dec	r18		;Zaehler inkrementieren bzw. dekrementieren
	inc	r17
	ldi	r16,EP0_FS-1	;Ueberpruefe ob FIFO gefuellt (hier 8 Byte)
	and	r16,r17		;Modulo durch UND-Vernuepfung mit 7!
	brne	SNDDE3
	lds	r16,UEINTX	;cbi UEINTX,TXINI (Bit 0)
	andi	r16,0xFE	;Paket abschicken
	sts	UEINTX,r16
SNDDE2:	lds	r16,UEINTX	;Warte bis TXINI wieder gesetzt (fertig, EP
	andi	r16,0x05	;Bank wieder frei) oder bis RXOUTI gesetzt
	breq	SNDDE2		;(ACK (ZLP) vom PC);
SNDDE3:	cpi	r18,0
	brne	SNDDE1
        ;Kontrolliere ob ACK(ZLP) vom PC angekommen (schon fertig)
        ;(kommt vor wenn Vielfaches des FIFO-Buffers n*8 angefragt waren)
	lds	r16,UEINTX	;Kontrolliere ob ACK (ZLP) vom PC
	andi	r16,0x04  	;angekommen (RXOUTI Flag gesetzt?)
	breq	SNDDE4
	rjmp	SNDDEA		;ACK senden und raus
SNDDE4: lds	r16,UEINTX	;cbi UEINTX,TXINI (Bit 0)
	andi	r16,0xFE	;Restpaket absenden (loescht EP Bank)
	sts	UEINTX,r16

SNDDE5:	lds	r16,UEINTX	;Warte bis RXOUTI wieder gesetzt
	andi	r16,0x04	;(ZLP (Erfolg) vom PC)
	breq	SNDDE5
SNDDEA:	lds	r16,UEINTX	;cbi UEINTX,RXOUTI (Bit 2)
	andi	r16,0xFB	;Falls ja dann Handshake (ACK), damit Int.
	sts	UEINTX,r16      ;bestaetigt

SNDDER:	pop	r17		;Register wiederherstellen
	pop 	r16
	ret

;-------------------------------------------------------------------------------
;       UP EP1_SP: RTCdaten einlesen und RTC setzen
;-------------------------------------------------------------------------------
EP1_SP:	push	r16		;rette Register (lokale Variablen)
	push	r17

	lds	r16,UEINTX	;Kontrolliere ob neue Daten (ZLP) vom PC
	andi	r16,0x04  	;(RXOUTI Flag gesetzt?)
	breq	EP1_SR

	lds	r16,UEINTX	;cbi UEINTX,RXOUTI (Bit 2)
	andi	r16,0xFB	;Falls ja dann Handshake (ACK),
	sts	UEINTX,r16      ;damit Interrupt bestaetigt
        lds	r16,UEINTX	;Kontrolliere ob Daten fuer Geraet im FIFO
	andi	r16,0x20  	;RWAL (Read/Write Allow Flag) gesetzt?
	breq	EP1_SR		;Ok Geraet darf lesen

	lds	r16,UEDATX      ;8 Byte einlesen und in Tabelle speichern
	sts	RTCTAB+0,r16
        lds     r16,UEDATX
        sts     RTCTAB+1,r16
        lds     r16,UEDATX
        sts     RTCTAB+2,r16
        lds     r16,UEDATX
        sts     RTCTAB+3,r16
        lds     r16,UEDATX
        sts     RTCTAB+4,r16
        lds     r16,UEDATX
        sts     RTCTAB+5,r16
        lds     r16,UEDATX
        sts     RTCTAB+6,r16
        lds     r16,UEDATX
        sts     RTCTAB+7,r16

        rcall   RTCSND

	lds     r16,UEINTX      ;cbi UEINTX,FIFOCON (Bit 7, FIFO Control Bit)
        andi    r16,0x7F        ;Melde, dass Speicherbank wieder frei.
        sts     UEINTX,r16

EP1_SR:	pop	r17             ;Register wiederherstellen
	pop	r16
	ret

;-------------------------------------------------------------------------------
;       UP EP2_AD: FIFO mit Sensorwerte fuettern
;-------------------------------------------------------------------------------
EP2_AD: push    r16             ;rette Register (lokale Variablen)

	lds	r16,UEINTX	;Kontrolliere ob Daten vom PC angefragt
	andi	r16,0x40  	;(NAK IN Flag gesetzt?)

	brne	EP2_A1
        rjmp    EP2_AR

EP2_A1: lds	r16,UEINTX	;cbi UEINTX,NAKINI (Bit 6)
	andi	r16,0xBF	;NAK_IN Flag soll von Software geloescht werden
	sts	UEINTX,r16

        lds     r16,UEINTX      ;Speicherbank frei?
        andi    r16,0x01        ;(Falls TXINI (fertig, EP Bank wieder frei)
        brne    EP2_A2          ;nicht gesetzt dann raus (siehe 22.14.1 S 275))
        rjmp    EP2_AR

EP2_A2: lds     r16,UEINTX	;cbi UEINTX,TXINI (Bit 0)
	andi	r16,0xFE	;sende ZLP Pak. (meldet Erfolg, loescht EP Bank)
	sts	UEINTX,r16


        lds     r16,RTCASCT3      ;Datum + Zeit (ASCII 16 Byte) abspeichern
        sts     UEDATX,r16
        lds     r16,RTCASCT3+1
        sts     UEDATX,r16
        ldi     r16,':'
        sts     UEDATX,r16
        lds     r16,RTCASCT3+2
        sts     UEDATX,r16
        lds     r16,RTCASCT3+3
        sts     UEDATX,r16
        ldi     r16,':'
        sts     UEDATX,r16
        lds     r16,RTCASCT3+4
        sts     UEDATX,r16
        lds     r16,RTCASCT3+5
        sts     UEDATX,r16

        lds     r16,RTCASCT3+6
        sts     UEDATX,r16
        lds     r16,RTCASCT3+7
        sts     UEDATX,r16
        ldi     r16,'/'
        sts     UEDATX,r16

        lds     r16,RTCASCT3+8
        sts     UEDATX,r16
        lds     r16,RTCASCT3+9
        sts     UEDATX,r16
        ldi     r16,'/'
        sts     UEDATX,r16

        lds     r16,RTCASCT3+10
        sts     UEDATX,r16
        lds     r16,RTCASCT3+11
        sts     UEDATX,r16


        lds     r16,CO2         ;CO2 Wert (ASCII 9 Byte) abspeichern
        sts     UEDATX,r16
        lds     r16,CO2+1
        sts     UEDATX,r16
        lds     r16,CO2+2
        sts     UEDATX,r16
        lds     r16,CO2+3
        sts     UEDATX,r16

        lds     r16,TEMP1         ;TEMP Wert (ASCII 10 Byte) abspeichern
        sts     UEDATX,r16
        lds     r16,TEMP1+1
        sts     UEDATX,r16
        lds     r16,TEMP1+2
        sts     UEDATX,r16
        lds     r16,TEMP1+3
        sts     UEDATX,r16
        lds     r16,TEMP1+4
        sts     UEDATX,r16
        lds     r16,TEMP1+5
        sts     UEDATX,r16

        lds     r16,HUM1         ;HUM Wert (ASCII 7 Byte) abspeichern
        sts     UEDATX,r16
        lds     r16,HUM1+1
        sts     UEDATX,r16
        lds     r16,HUM1+2
        sts     UEDATX,r16
        lds     r16,HUM1+3
        sts     UEDATX,r16
        lds     r16,HUM1+4
        sts     UEDATX,r16


        lds     r16,UEINTX      ;cbi UEINTX,FIFOCON (Bit 7, FIFO Control Bit)
        andi    r16,0x7F        ;Erlaube Abschicken der Daten durch Loeschen
        sts     UEINTX,r16      ;des FIFO Control Bit

EP2_AR: pop     r16             ;Register wiederherstellen
        ret

;;; hier erweitern falls weitere EPs benoetigt werden

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Deskriptoren im Programmspeicher (Flash, Word = 2Byte  LSB first!)
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
DEVDES:	.DB	18,0x01,0x10,0x01,0xFF,0xFF,0xFF,EP0_FS,0xeb,0x03,0x01,0x00
        .DB     0x01,0x00,MANU_I,PROD_I,SERI_I,0x01
		;bLength = 18 (0x12), Groesse des Deskriptors in Byte
		;bDescriptorType = 0x01, Konstante Device = 1
                ;bcdUSB = 0x0110, USB_Spec1_1
		;bDeviceClass = 0xFF, Klassencode: anbieterspez. (0xFF)
  		;bDeviceSubClass = 0xFF, Unterklassencode anbieters. (0xFF)
  		;bDeviceProtocoll = 0xFF, Protokollcode anbieterspez. (0xFF)
  		;bMaxPacketSize0 = EP0FS, max. Paketgroesse EP0 (hier 8 B)
		;idVendor = 0x03eb, Atmel Code durch usb.org vergeben
  		;idProduct = 0x0001, Produkt ID beliebig
		;bcdDevice = 0x0001, Release Nummer Geraet
		;iManufacturer = Index fuer String-Deskriptor Hersteller
  		;iProduct = Index fuer String-Deskriptor	Produkt
  		;iSerialNumber = Index fuer String-Deskriptor Seriennummer
		;bNumConfigurations = 1, Anzahl moeglicher Konfigurationen

CONDES:	.DB	9,0x02,9+9+(7*NR_EPS),0x00,0x01,0x01,0,0x80,250,9
        .DB     0x04,0,0,NR_EPS,0xFF,0xFF,0xFF,0,7,0x05,0x01,0x02
        .DB     LOW(EP1_FS),HIGH(EP1_FS),0,7,0x05,0x82,0x02,LOW(EP2_FS)
        .DB     HIGH(EP2_FS),0

        .	;;; hier erweitern falls weitere EPs benoetigt werden

;Config. Des.   ;bLength = 9, Groesse Deskriptor in Byte
		;bDescriptorType = 0x02, Konstante Configuration = 2
                ;wTotalLength, Laenge Configuration Deskriptor + alle
                ;untergeordnete Deskr.
		;bNumInterfaces = 1
  		;bConfigurationValue = 1, darf nicht Null sein (sonst geht
		;Geraet in den nicht-konfigurierten Zustand))
  		;iConfiguration = 0, Index fuer Str.-Deskr. Conf. (0 = kein Txt)
  		;bmAttributes = 0x80,bus-powered, kein remote wakeup Bit 7=1
  		;MaxPower = 50(dezimal), in 2mA Schritten! (100mA)
;Interface Des.: bLength = 9, Groesse Deskriptor in Byte
		;bDescriptorType = 0x04, Konstante Interface = 4
  		;bInterfaceNumber = 0;
  		;bAlternateSetting = 0;
  		;bNumEndpoints = USB_Endpoints;
  		;bInterfaceClass = 0xFF, Klassencode: anbieterspez. (0xFF)
  		;bInterfaceSubClass = 0xFF, Unterklassencode anbietersp. (0xFF)
  		;bInterfaceProtocol = 0xFF, Protokollcode anbieterspez. (0xFF)
  		;iInterface = 0, Index fuer String-Deskr. Interf. (0 = kein Txt)
;EP1 Deskr.:	 bLength = 7, Groesse Deskriptor in Byte
  		;bDescriptorType = 0x05, Konstante Endpoint = 5
		;bEndpointAddress = 0x01, Bit 7 = 0 (OUT) Bit 0-3 EP-Nummer
  		;bmAttributes = 0x02, Transfertyp = Bulk (2) (c=0, iso=1, int=3)
		;wMaxPacketSize = EP1_FIFO_Size;
		;bInterval = 0, Bulk EP, kein polling
;EP2 Deskr.:     bLength = 7, Groesse Deskriptor in Byte
 		;bDescriptorType = 0x05, Konstante Endpoint = 5
		;bEndpointAddress = 0x82, Bit 7 = 1 (IN) Bit 0-3 EP-Nummer
  		;bmAttributes = 0x02, Transfertyp = Bulk (2) (c=0, iso=1, int=3)
		;wMaxPacketSize = EP2_FIFO_Size;
		;bInterval = 0, Bulk EP, kein polling
ST0DES:	.DB     4,0x03,0x09,0x04	;Language String Deskriptor
		;bLength = 4, Groesse Deskriptor in Byte
  		;bDescriptorType = 0x03, Konstante String = 3
		;wLANGID[0] = 0x0409 = English USA (Supported Lang. Code 0)
ST1DES:	.DB     18,0x03         	;Manufacturer String Deskriptor
        .DB     'W',0,'E',0,'I',0,'G',0,'U',0,'.',0,'L',0,'U',0
		;bLength = 18, Groesse Deskriptor in Byte
  		;bDescriptorType = 0x03, Konstante String = 3
		;bString = Unicode Encoded String (16 Bit!)
ST2DES:	.DB     22,0x03         	;Product String Deskriptor
        .DB     'T',0,'E',0,'E',0,'N',0,'S',0,'Y',0,' ',0,'2',0,'.',0,'0',0
		;bLength = 22, Groesse Deskriptor in Byte
  		;bDescriptorType = 0x03, Konstante String = 3
		;bString = Unicode Encoded String (16 Bit!)
ST3DES:	.DB     10,0x03         	;Serial Number String Deskriptor
        .DB     'C',0,'O',0,'2',0,'2',0
		;bLength = 10, Groesse Deskriptor in Byte
  		;bDescriptorType = 0x03, Konstante String = 3
                ;bString = Unicode Encoded String (16 Bit!)

