;*******************************************************************************
;*
;*      Titel:	Hauptrogramm zur Nutzung der mobilen Messstation CO22
;*              (co22_mobil_weigu_firmware_v1_1_libusb.asm)
;*	Datum:	10/07/10			Version:	1.1 (8/12/12)
;*	Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*      
;*	Prozessor:   ATmega32u4	      Quarzfrequenz: 16 MHz
;*      
;*	Eingaenge: 
;*	Ausgaenge: 
;*                  
;*	Informationen zur Funktionsweise: 
;*      
;*      Dokumentation auf www.weigu.lu/b/co22
;*      
;*      Copyright (c) 2009   Guy WEILER         weigu[at]weigu[dot]lu
;*      
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*      
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*
;*******************************************************************************

;-------------------------------------------------------------------------------
;	Einbinden der controllerspezifischen Definitionsdatei
;-------------------------------------------------------------------------------
.NOLIST				;List-Output ausschalten
.INCLUDE "m32U4def.inc"         ;AVR-Definitionsdatei einbinden
;folgende Zeilen fehlen in der Defenitionsdatei!
;Bugreport wurde von ATMEL bestaetigt
.equ    TWAMR   = 0xbd  ; MEMORY MAPPED
.equ    TWCR    = 0xbc  ; MEMORY MAPPED
.equ    TWDR    = 0xbb  ; MEMORY MAPPED
.equ    TWAR    = 0xba  ; MEMORY MAPPED
.equ    TWSR    = 0xb9  ; MEMORY MAPPED
.equ    TWBR    = 0xb8  ; MEMORY MAPPED
; ***** TWI **************************
; TWAMR - TWI (Slave) Address Mask Register
.equ    TWAM0   = 1     ;
.equ    TWAMR0  = TWAM0 ; For compatibility
.equ    TWAM1   = 2     ;
.equ    TWAMR1  = TWAM1 ; For compatibility
.equ    TWAM2   = 3     ;
.equ    TWAMR2  = TWAM2 ; For compatibility
.equ    TWAM3   = 4     ;
.equ    TWAMR3  = TWAM3 ; For compatibility
.equ    TWAM4   = 5     ;
.equ    TWAMR4  = TWAM4 ; For compatibility
.equ    TWAM5   = 6     ;
.equ    TWAMR5  = TWAM5 ; For compatibility
.equ    TWAM6   = 7     ;
.equ    TWAMR6  = TWAM6 ; For compatibility

; TWBR - TWI Bit Rate register
.equ    TWBR0   = 0     ;
.equ    TWBR1   = 1     ;
.equ    TWBR2   = 2     ;
.equ    TWBR3   = 3     ;
.equ    TWBR4   = 4     ;
.equ    TWBR5   = 5     ;
.equ    TWBR6   = 6     ;
.equ    TWBR7   = 7     ;

; TWCR - TWI Control Register
.equ    TWIE    = 0     ; TWI Interrupt Enable
.equ    TWEN    = 2     ; TWI Enable Bit
.equ    TWWC    = 3     ; TWI Write Collition Flag
.equ    TWSTO   = 4     ; TWI Stop Condition Bit
.equ    TWSTA   = 5     ; TWI Start Condition Bit
.equ    TWEA    = 6     ; TWI Enable Acknowledge Bit
.equ    TWINT   = 7     ; TWI Interrupt Flag

; TWSR - TWI Status Register
.equ    TWPS0   = 0     ; TWI Prescaler
.equ    TWPS1   = 1     ; TWI Prescaler
.equ    TWS3    = 3     ; TWI Status
.equ    TWS4    = 4     ; TWI Status
.equ    TWS5    = 5     ; TWI Status
.equ    TWS6    = 6     ; TWI Status
.equ    TWS7    = 7     ; TWI Status

; TWDR - TWI Data register
.equ    TWD0    = 0     ; TWI Data Register Bit 0
.equ    TWD1    = 1     ; TWI Data Register Bit 1
.equ    TWD2    = 2     ; TWI Data Register Bit 2
.equ    TWD3    = 3     ; TWI Data Register Bit 3
.equ    TWD4    = 4     ; TWI Data Register Bit 4
.equ    TWD5    = 5     ; TWI Data Register Bit 5
.equ    TWD6    = 6     ; TWI Data Register Bit 6
.equ    TWD7    = 7     ; TWI Data Register Bit 7

; TWAR - TWI (Slave) Address register
.equ    TWGCE   = 0     ; TWI General Call Recognition Enable Bit
.equ    TWA0    = 1     ; TWI (Slave) Address register Bit 0
.equ    TWA1    = 2     ; TWI (Slave) Address register Bit 1
.equ    TWA2    = 3     ; TWI (Slave) Address register Bit 2
.equ    TWA3    = 4     ; TWI (Slave) Address register Bit 3
.equ    TWA4    = 5     ; TWI (Slave) Address register Bit 4
.equ    TWA5    = 6     ; TWI (Slave) Address register Bit 5
.equ    TWA6    = 7     ; TWI (Slave) Address register Bit 6

.LIST			        ;List-Output wieder einschalten

;-------------------------------------------------------------------------------
;       Organisation des Datenspeichers (SRAM)
;-------------------------------------------------------------------------------
.DSEG                           ;was ab hier folgt kommt in den SRAM-Speicher
.ORG    SRAM_START
;Globale Variable
ERR:    .BYTE   1               ;ERRorbyte
BRDIV:  .BYTE   3               ;DIVisor für BaudRate (1. Byte in BRDIV)
FWVER:  .BYTE   12              ;FirmWare VERsion (MAIN )dd(.)ddAAAAA
                                ;                 (RPRG )d(.)dd(R)
WRFNB:  .BYTE   1               ;Number of Bytes to write for WRF command
                                ;(max 256 = 0xFF)
RDFNB:  .BYTE   1               ;Number of Bytes to read for WRF command
                                ;(max 256 = 0xFF)
FNAME:  .BYTE   13              ;Filename max. 8+1(dot)+3+1(cr)
FSIZE:  .BYTE   4               ;Filesize 4 Bytes

BLINKFLAG:	.BYTE 1	        ;Blink On Off (0 = Off 1 = On)
HFLAG:          .BYTE 1         ;Flag, ob Kopfzeile zu schreiben ist

TEMP:           .BYTE 2         ;Rest + Quotient (fuer Temperaturkompensation)
TEMP1:          .BYTE 10        ;xxx,xx°C + Leerzeichen + Nullbyte
TEMP2:          .BYTE 6         ;xx°C + Leerzeichen + Nullbyte
SORH:           .BYTE 2         ;Rest + Quotient (fuer Temperaturkompensation)
TCFLAG:         .BYTE 1         ;Flag fuer Temperaturkompensation
TC:             .BYTE 6
HUM1:           .BYTE 7         ;xx,xx% + Nullbyte
HUM2:           .BYTE 4         ;xx% + Nullbyte

.ORG    0x0200
DIRLST: .BYTE   256             ;Nach einem DIR Kommando wird in DIRLST die
                                ;Anzahl der Dateien gespeichert
                                ;Die zurueckgegebenen ASCII-Zeichen werden ab
                                ;DIRLST+1 abgelegt.
WDATA:  .BYTE   256             ;Daten zum Schreiben in eine Datei
RDATA:  .BYTE   256             ;Daten zum Lesen aus einer Datei

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Programmspeicher (FLASH)     Programmstart nach RESET ab Adr. 0x0000
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.CSEG                           ;was ab hier folgt kommt in den FLASH-Speicher
.ORG	0x0000			;Programm beginnt an der FLASH-Adresse 0x0000
RESET1:	rjmp	INIT		;springe nach INIT (ueberspringe ISR Vektoren)

;-------------------------------------------------------------------------------
;	Sprungadressen fuer die Interrupts organisieren (ISR VECTORS)
;-------------------------------------------------------------------------------
        ;Vektortabelle (im Flash-Speicher)
.ORG    PCI0addr                ;interner Vektor fuer PCINT6 (1 Sekundentakt)
        rjmp    ISRPCS          ;Springe zur ISR des externen Interrupt
.ORG    USB_GENaddr             ;interner Vektor fuer USB General Interrupt
        rjmp    ISRGEN          ;Springe zur ISR des GENeral Interrupt
.ORG    USB_COMaddr             ;interner Vektor fuer USB Endpoint/Pipe
                                ;COMmunication Interrupt
        rjmp    ISRCOM          ;Springe zur ISR des EP COMmunication Interrupt


;-------------------------------------------------------------------------------
;	Initialisierungen und eigene Definitionen
;-------------------------------------------------------------------------------
.ORG    INT_VECTORS_SIZE        ;Platz fuer ISR Vektoren lassen
INIT:
.DEF    Zero = r15              ;Register 1 wird zum Rechnen benoetigt
        clr     r15             ;und mit Null belegt
.DEF	Tmp1 = r16		;Register 16 dient als erster Zwischenspeicher
.DEF	Tmp2 = r17		;Register 17 dient als zweiter Zwischenspeicher
.DEF    Cnt1 = r18              ;globale Zaehlvariable
.DEF    Flag = r19              ;globale Flagvariable
.DEF    WL = r24                ;Register 24 und 25 dienen als universelles
.DEF    WH = r25                ;Doppelregister W

	;Stapel initialisieren (fuer Unterprogramme bzw. Interrupts)
	ldi	Tmp1,HIGH(RAMEND)	;RAMEND (SRAM) ist in der Definitions-
	out	SPH,Tmp1		;datei festgelegt
	ldi	Tmp1,LOW(RAMEND)
	out	SPL,Tmp1

	;Wirkung des Fusebit CKDIV ueberschreiben
        ldi     r16,0x80        ;CLKPCE = 1 Wirkung des Fusebit CKDIV
        sts     CLKPR,r16       ;ueberschreiben
        clr     r16             ;CLKPR = 0
        sts     CLKPR,r16

        ;Ausgaenge initialisieren
        sbi     DDRD,6          ;Teensy LED!
        sbi     DDRD,7          ;Summer!
        sbi     DDRF,6          ;LED rot
        sbi     DDRF,5          ;LED gelb
        sbi     DDRF,4          ;LED gruen
        sbi     PORTD,6         ;Teensy LED ein

        ;LCD-Anzeige initialisieren und Text ausgeben
        rcall   LCDINI
        
        ldi     r24,0x80		;1 Zeile 1 Stelle
        rcall   LCDSCO
        ldi     ZH,HIGH(TEXT4*2) ;Z-Ptr zeigt auf Textanfang Flash
        ldi     ZL,LOW(TEXT4*2)
        rcall   LCDSTF          ;gibt FlashText
        ldi     r24,0xC0		;2 Zeile 1 Stelle
        rcall   LCDSCO
        ldi     ZH,HIGH(TEXT5*2) ;Z-Ptr zeigt auf Textanfang Flash
        ldi     ZL,LOW(TEXT5*2)
        rcall   LCDSTF          ;gibt FlashText

        ;3 Sekunden warten bis VDrive startbereit
        clr     XH              ;
        ldi     XL,3            ;
        rcall   WX1s            ;
        cbi     PORTD,6         ;Teensy LED aus

        ;USART initialisieren
        ;Sender einschalten und Datenformat festlegen
        lds     Tmp1,UCSR1B
        ori     Tmp1,0x18       ;Sender + Empfaenger einschalten
        sts     UCSR1B,Tmp1
        ldi     Tmp1,0x06       ;UCSR1C = 00000110b
        sts     UCSR1C,Tmp1     ;Waehle Asynchr. 8N1
        ;Baudrate initialisieren mit Defaultbaudrate von 9600 Bit/s
        clr     Tmp1            ;UBRR = 103 = 0x0067
        sts     UBRR1H,Tmp1     
        ldi     Tmp1,103        
        sts     UBRR1L,Tmp1     

        clr     Tmp1
        sts     ERR,Tmp1         ;ERRorbyte loeschen

	;ADC initialisieren
        ldi     Tmp1,0b11000000	;REFS = 11 Interne 2,56V Referenz
        sts     ADMUX,Tmp1      ;ADLAR = 0 rechtsbuendig
                                ;MUX = 00000 unsymetrisch PF0 (ADC0)
	ldi     Tmp1,0b10000111 ;ADEN = 1 (ADC einschalten)
        sts     ADCSRA,Tmp1     ;ADPS = 111 (16MHz/128 = 125kHz)
                                ;andere Bits per default = 0

        ;externen PCINT Interrupt initialisieren (Sekundentakt vom RTC)
        cbi     DDRB,6
        sbi     PORTB,6         ;Setze Pull-Up (wird von RTC benoetigt!)
        ldi     Tmp1,0x40       ;PCINT6 an PB6
        sts     PCMSK0,Tmp1

	;globale Variable initialisieren
        clr     Tmp1		;Kopfzeilenflag = 0
        sts     HFLAG,Tmp1
        clr	Flag		;Interruptflag loeschen

        ;Default RAM Werte fuer CO2, Temperatur und Feuchtigkeit
        	ldi	Tmp1,'0'
        sts     CO2+0,Tmp1
        	ldi	Tmp1,'0'
        sts     CO2+1,Tmp1
        	ldi	Tmp1,'0'
        sts     CO2+2,Tmp1
        	ldi	Tmp1,'0'
        sts     CO2+3,Tmp1
        	ldi	Tmp1,'p'
        sts     CO2+4,Tmp1
        	ldi	Tmp1,'p'
        sts     CO2+5,Tmp1
        ldi     Tmp1,'m'
        sts     CO2+6,Tmp1
        ldi     Tmp1,' '         ;Leerzeichen
        sts     CO2+7,Tmp1
        clr     Tmp1             ;Nullbyte
        sts     CO2+8,Tmp1
	ldi	Tmp1,'0'
        sts     TEMP2+0,Tmp1
        	ldi	Tmp1,'0'
        sts     TEMP2+1,Tmp1
        ldi     Tmp1,0xDF	;Gradzeichen LCD
        sts     TEMP2+2,Tmp1
        ldi     Tmp1,'C'		;C
        sts     TEMP2+3,Tmp1
        ldi     Tmp1,' '         ;Leerzeichen
        sts     TEMP2+4,Tmp1
        clr     Tmp1             ;Nullbyte
        sts     TEMP2+5,Tmp1
	ldi	Tmp1,'0'
        sts     HUM2+0,Tmp1
        	ldi	Tmp1,'0'
        sts     HUM2+1,Tmp1
        ldi     Tmp1,'%'		;Prozentzeichen
        sts     HUM2+2,Tmp1
        clr     Tmp1             ;Nullbyte
        sts     HUM2+3,Tmp1


;-------------------------------------------------------------------------------
;	Hauptprogramm
;-------------------------------------------------------------------------------
MAIN:   rcall   INIDEV          ;befindet sich in USB_DRV.asm
	sei
	

        rcall   I2CINI

        call    SCS             ;Kurzbefehlsmodus

	;Kontrolle ob die Uhr schon laeuft sonst starten!
        call    RTCRCV		;RTC auslesen
        call    RTCASC		;nach ASCII
        lds	Tmp1,RTCTAB+0
        rcall	W1s
        rcall	W1ms
        call    RTCRCV		;RTC auslesen
        call    RTCASC		;nach ASCII
        lds	Tmp2,RTCTAB+0
        cp	Tmp1,Tmp2
        brne	MAIN2
        call	RTCINI

MAIN2:	call	RTCSQW		;SQW Ausgang des RTC freischalten (PCINT ISR)

        call    RTCRCV		;RTC auslesen
        call    RTCASC		;nach ASCII


        ;Dateiname mit Datum erstellen
        ldi     YL,LOW(FNAME)   ;DateiName
        ldi     YH,HIGH(FNAME)  ;1. Zeiger initialisieren (SRAM)
        ldi     ZL,LOW(FNAMEF*2) ;2. Zeiger initialisieren (Flash)
        ldi     ZH,HIGH(FNAMEF*2)
DNAMEL: lpm     Tmp1,Z+         ;Dateinamen kopieren von Flash nach SRAM
        st      Y+,Tmp1
        cpi     Tmp1,0x0D
        brne    DNAMEL
        lds     Tmp1,RTCASCT3+6   ;Datum einfuegen
        sts     FNAME+0,Tmp1
        lds     Tmp1,RTCASCT3+7
        sts     FNAME+1,Tmp1
        lds     Tmp1,RTCASCT3+8
        sts     FNAME+3,Tmp1
        lds     Tmp1,RTCASCT3+9
        sts     FNAME+4,Tmp1
        lds     Tmp1,RTCASCT3+10
        sts     FNAME+6,Tmp1
        lds     Tmp1,RTCASCT3+11
        sts     FNAME+7,Tmp1





        ldi     Tmp1,0x4E       ;Aendere die Baudrate VDrive2 auf 38400 bit/s
        sts     BRDIV,Tmp1      ;(1 Byte = 0x4E; 2 Byte = 0xC0 ;3 Byte = 0x00)
        ldi     Tmp1,0xC0
        sts     BRDIV+1,Tmp1
        clr     Tmp1
        sts     BRDIV+2,Tmp1
        call    SBD
        clr     Tmp1            ;Baudrate ATmega auf 38400 bit/s
        sts     UBRR1H,Tmp1     ; (Quarz 16Mhz!)
        ldi     Tmp1,25
        sts     UBRR1L,Tmp1     ;Teiler bei 16MHz = 25
        
        ldi     Tmp1,0x01       ;Erlaube PCINT Interrupt
        sts     PCICR,Tmp1


MLOOP:  sbi	PORTF,4
	rcall   CO2RD           ;CO2 Sensor lesen
        rcall   SHTRTE          ;Temperatur lesen
        rcall   SHTRHU          ;Luftfeuchtigkeit lesen
        cbi	PORTF,4
        cpi	Flag,0xFF
        brne	MLOOP
        sbi	PORTF,6
        in	Tmp1,PIND	;Toggle LED
        ldi	Tmp2,0x40	;PD6 Teensy LED
        eor	Tmp1,Tmp2
        out	PORTD,Tmp1
        call    RTCRCV		;RTC auslesen
        call    RTCASC		;nach ASCII
        rcall	LCD
	rcall	STICK
        clr 	Flag
        cbi	PORTF,6
        rjmp    MLOOP

;-------------------------------------------------------------------------------
;	Unterprogramme und Interrupt-Behandlungsroutinen
;-------------------------------------------------------------------------------
;ISR: Aufruf durch wechselnde Flanke an PB6
;RTC liefert das Rechtecksignal mit 1Hz
;ISR wird 2* pro Sekunde aufgerufen!
ISRPCS: push    r16
        in      r16,SREG
        push    r16

        sbi	PORTF,5

        sbis    PINB,6
        rjmp    ISRPCR

	ldi	Flag,0xFF

	cbi	PORTF,5

ISRPCR: pop     r16
        out     SREG,r16
        pop     r16
        reti

; Ausgabe der Daten an das LCD       
LCD:	push	r24
	push	ZL
	push	ZH

	;Ausgabe LCD Text 1 Zeile (Uhrzeit Datum)
        ldi     r24,0x80		;1 Zeile 1 Stelle
        rcall   LCDSCO
        ldi     r24,' '		;Leerzeichen
        rcall   LCDOUT
        ldi     r24,0x81		;1 Zeile 2 Stelle
        rcall   LCDSCO
        ldi     ZH,HIGH(RTCASCT2) ;Z-Ptr zeigt auf Textanfang RAM
        ldi     ZL,LOW(RTCASCT2)
        rcall   LCDSTR          ;gibt RAMText aus

        ;Ausgabe LCD Text 2 Zeile (Sensordaten)
        ldi     r24,0xC0        ;2 Zeile 1 Stelle
        rcall   LCDSCO
        ldi     ZH,HIGH(CO2)    ;Z-Ptr zeigt auf Textanfang RAM CO2
        ldi     ZL,LOW(CO2)
        rcall   LCDSTR          ;gibt RAMText aus
        ldi     r24,0xC8        ;2 Zeile 9 Stelle
        rcall   LCDSCO
        ldi     ZH,HIGH(TEMP2)	;Z-Ptr zeigt auf Textanfang RAM Temperatur
        ldi     ZL,LOW(TEMP2)
        rcall   LCDSTR          ;gibt RAMText aus
        ldi     r24,0xCD        ;2 Zeile 14 Stelle
        rcall   LCDSCO
        ldi     ZH,HIGH(HUM2)   ;Z-Ptr zeigt auf Textanfang RAM Luftfeuchtigkeit
        ldi     ZL,LOW(HUM2)
        rcall   LCDSTR          ;gibt RAMText aus

	pop	ZH
	pop	ZL
	pop	r24
	ret

;USB-Stick alle 5 Sekunden beschreiben
STICK:	push    r16
	push	r18
        push    r24
        push    r25
        push    XL
        push    XH
        push    YL
        push    YH
        push    ZL
        push    ZH

        call    TSTIFD          ;Ist USB-Stick vorhanden?
        cpi     r24,0           ;Falls ja, dann weiter
        brne    STICK1
        clr     r16             ;Setze Kopfzeilen-Flag zurueck
        sts     HFLAG,r16
STICK0: rjmp    STICKR
STICK1: lds     r16,RTCASCT3+5    ;Sekunden laden
        cpi     r16,'0'         ;Falls nicht 5 Sekunde, dann raus
        breq    STICK2
        cpi     r16,'5'
        brne    STICK0
STICK2: lds     r16,HFLAG
        cpi     r16,1
        breq    STICKW          ;Falls T-Flag, dann keine Kopfzeile
        ;Erstelle Datei und schreibe Header (nur 1x)
        ldi     XL,LOW(WDATA)   ;Kopfzeile schreiben
        ldi     XH,HIGH(WDATA)  ;1. Zeiger initialisieren
        ldi     ZL,LOW(HEADL*2) ;2. Zeiger initialisieren
        ldi     ZH,HIGH(HEADL*2)
        clr     r18
STICKL: lpm     r16,Z+         ;Kopfzeile kopieren vom Flash ins SRAM
        st      X+,r16
        cpi     r16,0
        breq    STICK3
        inc     r18
        rjmp    STICKL

        	sbi	PORTF,6
STICK3: call    OPW             ;OPW
        sts     WRFNB,r18
        call    WRF             ;WRF
        call    CLF             ;CLF
        ldi     r16,1           ;Setze Kopfzeilen-Flag
        sts     HFLAG,r16
STICKW:   ;Datum und Uhrzeit
        lds     r16,RTCASCT3+6   ;Datum
        sts     WDATA+0,r16
        lds     r16,RTCASCT3+7
        sts     WDATA+1,r16
        ldi     r16,'.'
        sts     WDATA+2,r16
        lds     r16,RTCASCT3+8
        sts     WDATA+3,r16
        lds     r16,RTCASCT3+9
        sts     WDATA+4,r16
        ldi     r16,'.'
        sts     WDATA+5,r16
        lds     r16,RTCASCT3+10
        sts     WDATA+6,r16
        lds     r16,RTCASCT3+11
        sts     WDATA+7,r16
        ldi     r16,0x09       ;TAB
        sts     WDATA+8,r16
        lds     r16,RTCASCT3+0   ;Uhrzeit
        sts     WDATA+9,r16
        lds     r16,RTCASCT3+1
        sts     WDATA+10,r16
        ldi     r16,':'
        sts     WDATA+11,r16
        lds     r16,RTCASCT3+2
        sts     WDATA+12,r16
        lds     r16,RTCASCT3+3
        sts     WDATA+13,r16
        ldi     r16,':'
        sts     WDATA+14,r16
        lds     r16,RTCASCT3+4
        sts     WDATA+15,r16
        lds     r16,RTCASCT3+5
        sts     WDATA+16,r16
        ldi     r16,0x09       ;TAB
        sts     WDATA+17,r16
        ;Temperatur
        ldi     XL,LOW(WDATA+18)  ;1. Zeiger initialisieren
        ldi     XH,HIGH(WDATA+18) ;zu schreibende Daten (SRAM)
        ldi     YL,LOW(TEMP1)   ;2. Zeiger initialisieren
        ldi     YH,HIGH(TEMP1)  ;Temperatur (SRAM)
STICKT: ld      r16,Y+         ;Temperatur TEMP1 (SRAM) -> WDATA (SRAM)
        st      X+,r16
        cpi     r16,0xDF       ;Gradzeichen
        brne    STICKT
        ldi     r16,0x09       ;(TAB) Right Tab.
        sts     WDATA+24,r16
        ;Luftfeuchtigkeit
        ldi     XL,LOW(WDATA+25)  ;1. Zeiger initialisieren
        ldi     XH,HIGH(WDATA+25) ;zu schreibende Daten (SRAM)
        ldi     YL,LOW(HUM1)   ;2. Zeiger initialisieren
        ldi     YH,HIGH(HUM1)  ;Temperatur (SRAM)
STICKH: ld      r16,Y+        ;Temperatur TEMP1 (SRAM) -> WDATA (SRAM)
        st      X+,r16
        cpi     r16,'%'
        brne    STICKH
        ldi     r16,0x09      ;(TAB) Right Tab.
        sts     WDATA+30,r16
        ;CO2
        ldi     XL,LOW(WDATA+31)  ;1. Zeiger initialisieren
        ldi     XH,HIGH(WDATA+31) ;zu schreibende Daten (SRAM)
        ldi     YL,LOW(CO2)    ;2. Zeiger initialisieren
        ldi     YH,HIGH(CO2)   ;Temperatur (SRAM)
STICKC: ld      r16,Y+        ;Temperatur TEMP1 (SRAM) -> WDATA (SRAM)
        st      X+,r16
        cpi     r16,'p'
        brne    STICKC

        ldi     r16,0x0D      ;CR
        sts     WDATA+35,r16

        call    OPW            ;OPW
        ldi     r16,36         ;Es sollen 36 Zeichen geschrieben werden!
        sts     WRFNB,r16
        call    WRF            ;WRF
        call    CLF            ;CLF

STICKR:	pop     ZH
        pop     ZL
        pop     YH
        pop     YL
        pop     XH
        pop     XL
        pop     r25
        pop     r24
        pop	r18
        pop     r16
        ret

.INCLUDE "SR_TIME_16M.asm" 	 	;Zeitschleifen Bibliothek
.INCLUDE "SR_USB_32u4_CO22.asm" 	;USB Bibliothek
.INCLUDE "SR_LCD_2x16_4Bit.asm" 		;LCD Bibliothek
.INCLUDE "SR_I2C.asm"			;I2C Bibliothek
.INCLUDE "SR_CO2.asm"           		;CO2 Bibliothek
.INCLUDE "SR_SHT75.asm"         		;SHT75 (Temp und Feuchtigkeit) Bibliothek
.INCLUDE "SR_RTC_DS1307.asm"    		;RTC Bibliothek
.INCLUDE "SR_RECODE.asm"     		;Kodier Bibliothek
.INCLUDE "SR_MUL_DIV.asm"       		;Rechen Bibliothek
.INCLUDE "SR_VDRIVE2_32u4_CO22.asm"  	;VDrive Bibliothek

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EXIT				;Ende des Quelltextes
